###########################################################################
#
#   Copyright (c) 2001 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2001 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $

__doc__ = 'Value checker and processor'
__version__ = '$Revision 1.0 $'

import Globals
import ZPublisher
import string
import convert_integer
import convert_float
import convert_required
import convert_empty_as_none
import convert_list
import convert_text
import convert_string
import convert_boolean
import convert_email
import build_datetime

value_checkers = {}

message_dialogue = Globals.HTMLFile('message_dialogue', globals())

http_request_http_request_process_inputs = ZPublisher.HTTPRequest.HTTPRequest.processInputs

def register_value_checker(name, module):
	"""This let's us add value checkers from other modules."""

	value_checkers[name] = module


register_value_checker('integer', convert_integer)
# Don't like it one bit.
register_value_checker('int', convert_integer)
register_value_checker('required', convert_required)
register_value_checker('float', convert_float)
register_value_checker('list', convert_list)
register_value_checker('text', convert_text)
register_value_checker('string', convert_string)
register_value_checker('boolean', convert_boolean)
register_value_checker('emtpy_as_none', convert_empty_as_none)
register_value_checker('build_datetime', build_datetime)
register_value_checker('email', convert_email)


def process_inputs(request):
	"""Method that hooks itself onto the standard form data processing."""

	# We don't want to disturb anybody

	http_request_http_request_process_inputs(request)

	ignore_conversion_error = request.form.has_key('__ignore_conversion_error')
	ignore_unknown_value_checker = request.form.has_key('__ignore_unknown_value_checker')

	# How can we can ahold of the object being published?
	# Seems we have to do something that they do in the ZPublisher.Publish module..
	# Hmmmm.... =]

	published_object = None

	string_find = string.find
	string_split = string.split

	request.value_checker_exception_stack = []

	form_update = {}

	for key in request.form.keys():

		if string_find(key, ';') > -1:

			splits = string_split(key, ';')

			field_name = splits[0]

			for value_checker in splits[1:]:

				value = request.form[key]

				if value_checker in value_checkers.keys():

					value_checker = value_checkers[value_checker]

					try:

						# So we can check several conditions

						try:

							value = form_update[field_name]

						except KeyError:

							pass

						form_update[field_name] = value_checker.convert(value)

					except ValueError, value:

						if ignore_conversion_error:

							form_update[field_name] = value
							continue

						request.value_checker_exception_stack.append(
							{
								'field_name':field_name,
								'value':value,
								'help':value_checker.help
							})

						# We need to add some logic so that the user
						# doesn't have to go back, fill in the wrong value
						# and then go forward again, to then go back,
						# i.e. that the value needs to be integer and
						# that it is required, etc...

						break

				else:

					if ignore_unknown_value_checker:

						form_update[key] = value
						continue

					raise SyntaxError, 'Unknown value checker/converter: %s' % value_checker

	print request.form

	request.form = form_update

	if request.value_checker_exception_stack:

		if hasattr(published_object, 'standard_value_error_message'):

			published_object.standard_value_error_message(None, REQUEST=request, RESPONSE=request.RESPONSE)

		else:

			request.RESPONSE.write(message_dialogue(None, REQUEST=request, RESPONSE=request.RESPONSE))


ZPublisher.HTTPRequest.HTTPRequest.processInputs = process_inputs
